/*
 * Created on Apr 20, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.imaginot.chess;

import java.util.*;

import chesspresso.position.Position;
import chesspresso.*;

/**
 * @author Administrator
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class AbstractPositionGenerator implements PositionGenerator {
	public final static int LIMIT = 10000;
	protected Position pos = new Position();
	protected Position initialPos;
	protected List pieces;
	protected boolean whiteToMove;
	protected int[] indexes;

	public AbstractPositionGenerator(List pieces, boolean side) {
		setPieces(pieces);
		setWhiteToMove(side);
		setInitialPosition(new Position());
	}

	public AbstractPositionGenerator(
		List pieces,
		boolean side,
		Position init) {
		setPieces(pieces);
		setWhiteToMove(side);
		setInitialPosition(init);
	}

	private void setInitialPosition(Position p) {
		initialPos = p;
	}
	private boolean validPosition() {
		if (!pos.isLegal()) {
			return false;
		}
		return wantPosition();
	}
	private void nextIndexes() {
		for (int i = 0; i < indexes.length; i++) {
			indexes[i] = (indexes[i] + 1) % 64;
			if (indexes[i] != 0)
				break;
		}
	}
	/* (non-Javadoc)
	 * @see com.imaginot.chess.PositionGenerator#genPosition()
	 */
	public Position genPosition() {
		int count = 0;
		do {
			count++;
			do {

				nextIndexes();
				clearBoard();
				placePieces();
			} while (!pos.isLegal());

		}
		while (!validPosition());

		return pos;
	}

	/* (non-Javadoc)
	 * @see com.imaginot.chess.PositionGenerator#setPieces(java.util.List)
	 */
	public void setPieces(List pieces) {
		this.pieces = pieces;
		indexes = new int[pieces.size()];
		for (int i = 0; i < indexes.length; i++)
			indexes[i] = 0;
	}

	/* (non-Javadoc)
	 * @see com.imaginot.chess.PositionGenerator#setWhiteToMove(boolean)
	 */
	public void setWhiteToMove(boolean side) {
		whiteToMove = side;
	}

	protected void clearBoard() {
		pos = new Position(initialPos);
		pos.setToPlay(whiteToMove ? Chess.WHITE : Chess.BLACK);
	}

	protected void placePieces() {
		Iterator it = pieces.iterator();
		int pnum = 0;
		while (it.hasNext()) {
			int piece = ((Integer) it.next()).intValue();
			if ((piece == Chess.WHITE_PAWN) || (piece == Chess.BLACK_PAWN)) {
				pos.setStone(findPawnSquare(pnum), piece);
			} else {
				pos.setStone(findSquare(pnum), piece);
			}
			pnum++;
		}
	}

	protected abstract boolean wantPosition();

	protected int findPawnSquare(int pnum) {
		return findSquare(pnum);
	}

	protected int findSquare(int pnum) {
		int sq = indexes[pnum];
		for (int i = 0; i < 63; i++) {
			if (pos.getStone(sq) != Chess.NO_STONE) {
				sq = (sq + 1) % 64;
			} else {
				break;
			}
			//System.err.println("Got NO_STONE");
		}
		return sq;
	}
}
