package com.imaginot.chess;

import java.util.*;

public class Square {
	private long letter;
	private long rank;
	private char color;

	public Square() {
		int temp = rand(64);
		letter =
			temp % 8;
		rank =
			temp/8;

		if (((letter + rank) % 2) == 0) {
			color = 'w';
		} else {
			color = 'b';
		}
	}

	public Square(long letter, long rank) {
		this.letter = Math.abs(letter);
		this.rank = Math.abs(rank);
		if (((letter + rank) % 2) == 0) {
			color = 'w';
		} else {
			color = 'b';
		}
	}

	public char getColor() {
		return color;
	}

	public String colorName() {
		if (color == 'w') {
			return "white";
		} else {
			return "black";
		}
	}

	public boolean sameSquare(Square o) {
		return (this.letter == o.letter) && (this.rank == o.rank);
	}

	public boolean equals(Object o) {
		if (o instanceof Square) {
			return sameSquare((Square) o);
		} else {
			return false;
		}
	}

	public int hashCode() {
		return (new Long(letter + (13 * rank)).intValue());
	}

	public boolean isWhite() {
		return color == 'w';
	}

	public String toString() {
		return "" + (char) ((long) 'h' - letter) + "" + (rank + 1);
	}

	public boolean sameColor(Square o) {
		return this.color == o.color;
	}

	public Square genInstance() {
		Square o;
		o = new Square();
		if (sameSquare(o)) {
			o = new Square();
		}
		if (sameSquare(o)) {
			o = new Square();
		}
		if (sameSquare(o)) {
			o = new Square();
		}
		if (sameSquare(o)) {
			o = new Square((this.letter + 2) % 8, (this.rank+1)%8);
		}
		return o;
	}

	public Square genCloseInstance() {
		int ldelta = rand(3) - 1;
		int rdelta = rand(5) - 2;

		Square o =
			new Square((this.letter + ldelta) % 8, (this.rank + rdelta) % 8);
		if (sameSquare(o)) {
			o = new Square((this.letter + 1) % 8, this.rank);
		}
		return o;
	}
	
	public boolean sameDiagonal(Square o) {
		return (Math.abs(letter - o.letter) == Math.abs(rank - o.rank));
	}

	public boolean knightMove(Square o) {
		long over = Math.abs(letter - o.letter);
		long up = Math.abs(rank - o.rank);

		if ((over == 2) && (up == 1)) {
			return true;
		} else if ((over == 1) && (up == 2)) {
			return true;
		} else {
			return false;
		}
	}

	public boolean doubleKMove(Square o) {
		List l = genKnight();
		Iterator it = l.iterator();

		while (it.hasNext()) {
			Square sq = (Square) it.next();
			if (sq.knightMove(o)) {
				return true;
			}
		}
		return false;
	}

	public List genDiagonal(int hdir, int vdir) {
		hdir = (hdir > 0) ? 1 : -1;
		vdir = (vdir > 0) ? 1 : -1;

		ArrayList mv = new ArrayList();

		long lt = letter;
		long rk = rank;

		lt += hdir;
		rk += vdir;

		while ((lt < 8) && (lt >= 0) && (rk < 8) && (rk >= 0)) {
			mv.add(new Square(lt, rk));
			lt += hdir;
			rk += vdir;
		}

		lt = letter;
		rk = rank;

		lt -= hdir;
		rk -= vdir;

		while ((lt < 8) && (lt >= 0) && (rk < 8) && (rk >= 0)) {
			mv.add(new Square(lt, rk));
			lt -= hdir;
			rk -= vdir;
		}

		return mv;
	}

	public List genKnight() {
		ArrayList mv = new ArrayList();

		long lt;
		long rk;

		long dir1[] = { 2, 1 };
		long dir2[] = { 1, 2 };

		for (int i = 0; i < 4; i++) {
			long temp;
			lt = letter + dir1[0];
			rk = rank + dir1[1];

			if ((lt < 8) && (lt >= 0) && (rk < 8) && (rk >= 0)) {
				mv.add(new Square(lt, rk));
			}

			lt = letter + dir2[0];
			rk = rank + dir2[1];

			if ((lt < 8) && (lt >= 0) && (rk < 8) && (rk >= 0)) {
				mv.add(new Square(lt, rk));
			}
			// rotate through our perms
			temp = dir1[0];
			dir1[0] = dir1[1];
			dir1[1] = -temp;
			temp = dir2[0];
			dir2[0] = dir2[1];
			dir2[1] = -temp;
		}

		return mv;
	}

	public List genDoubleKnight() {
		Set mv = new HashSet();

		List moves = genKnight();
		Iterator it = moves.iterator();
		while (it.hasNext()) {
			Square sq = (Square) it.next();
			mv.addAll(sq.genKnight());
		}

		return new ArrayList(mv);
	}

	public static void main(String[] args) {
		Square sq1;
		Square sq2;

		for (int i = 0; i < 10; i++) {
			sq1 = new Square();
			sq2 = new Square();
			System.out.println("Square: " + sq1 + ", " + sq1.getColor());
			System.out.println("Square: " + sq2 + ", " + sq2.getColor());
			if (sq1.sameColor(sq2)) {
				System.out.println("Same color.");
			} else {
				System.out.println("Different color.");
			}
			if (sq1.sameDiagonal(sq2)) {
				System.out.println("Same diagonal.");
			} else {
				System.out.println("Different diagonal.");
			}
			if (sq1.knightMove(sq2)) {
				System.out.println("Is knight move.");
			} else {
				System.out.println("Not knight move.");
			}
			if (sq1.doubleKMove(sq2)) {
				System.out.println("Is double knight move.");
			} else {
				System.out.println("Not double knight move.");
			}

			List l = sq1.genDiagonal(1, 1);
			Iterator it = l.iterator();

			System.out.print("Diagonal for " + sq1 + ": ");
			while (it.hasNext()) {
				sq2 = (Square) it.next();
				System.out.print("" + sq2 + ", ");
			}
			System.out.println("");

			l = sq1.genKnight();
			it = l.iterator();

			System.out.print("Knight moves for " + sq1 + ": ");
			while (it.hasNext()) {
				sq2 = (Square) it.next();
				System.out.print("" + sq2 + ", ");
			}
			System.out.println("");
		}
	}

	public final static long BIGPRIME = 2038074743;
	public static long lrand(long n) {
		return (Math.round(Math.random() * BIGPRIME)) % n;
	}
	public static int rand(int n) {
		return (int) lrand(n);
	}
}
