/*
 * Created on Apr 14, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.imaginot.chess.swing;

import javax.swing.*;
import java.awt.*;

/**
 * @author dkappe
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ChessBoard extends JComponent implements ChessBoardListener {

	final public static int BOARD_DIM = 258;

	private ChessBoardModel model;
	private ChessIcons chessIcons;
	private boolean whiteBottom = true;
	private boolean hideNW = false;
	private boolean hideNE = false;
	private boolean hideSW = false;
	private boolean hideSE = false;
	private boolean hidePawns = false;
	private boolean hidePieces = false;

	public ChessBoard() {
		chessIcons = new ChessIcons();
		this.setMinimumSize(new Dimension(BOARD_DIM, BOARD_DIM));
		this.setMaximumSize(new Dimension(BOARD_DIM, BOARD_DIM));
		this.setPreferredSize(new Dimension(BOARD_DIM, BOARD_DIM));
		this.setModel(
			new GenericChessBoardModel(GenericChessBoardModel.START_POSITION));
	}

	public ChessBoard(ChessBoardModel m) {
		chessIcons = new ChessIcons();
		this.setMinimumSize(new Dimension(BOARD_DIM, BOARD_DIM));
		this.setMaximumSize(new Dimension(BOARD_DIM, BOARD_DIM));
		this.setPreferredSize(new Dimension(BOARD_DIM, BOARD_DIM));
		this.setModel(m);
	}

	public void setModel(ChessBoardModel m) {
		if (model != null) {
			model.removeChessBoardListener(this);
		}
		model = m;
		model.addChessBoardListener(this);
	}

	public void paintComponent(Graphics g) {
		//	   ensure proper background handling
		//if (isOpaque()) {
		//	super.paintComponent(g);
		//}
		g.drawImage(chessIcons.board.getImage(), 1, 1, this);
		g.drawRect(0,0,257,257);
		if (model != null) {
			for (int i = 7; i >= 0; i--) {
				for (int j = 0; j < 8; j++) {
					int mapi;
					int mapj;

					if (whiteBottom) {
						mapi = i;
						mapj = j;
					} else {
						mapi = 7 - i;
						mapj = 7 - j;
					}

					int y = (7 - i) * 32 + 1;
					int x = j * 32 + 1;
					char[][] board = model.getBoard();
					ImageIcon icon;
					switch (board[mapi][mapj]) {
						case 'R' :
							icon = hidePieces?null:chessIcons.whiteRook;
							break;
						case 'N' :
							icon = hidePieces?null:chessIcons.whiteKnight;
							break;
						case 'B' :
							icon = hidePieces?null:chessIcons.whiteBishop;
							break;
						case 'Q' :
							icon = hidePieces?null:chessIcons.whiteQueen;
							break;
						case 'K' :
							icon = hidePieces?null:chessIcons.whiteKing;
							break;
						case 'P' :
							icon = hidePawns?null:chessIcons.whitePawn;
							break;
						case 'r' :
							icon = hidePieces?null:chessIcons.blackRook;
							break;
						case 'n' :
							icon = hidePieces?null:chessIcons.blackKnight;
							break;
						case 'b' :
							icon = hidePieces?null:chessIcons.blackBishop;
							break;
						case 'q' :
							icon = hidePieces?null:chessIcons.blackQueen;
							break;
						case 'k' :
							icon = hidePieces?null:chessIcons.blackKing;
							break;
						case 'p' :
							icon = hidePawns?null:chessIcons.blackPawn;
							break;
						default :
							icon = null;
					}
					if ((i > 3) && (j < 4) && hideNW) {
						// hide the NW
						icon = null;
					}
					if ((i > 3) && (j > 3) && hideNE) {
						// hide the NW
						icon = null;
					}
					if ((i < 4) && (j < 4) && hideSW) {
						// hide the NW
						icon = null;
					}
					if ((i < 4) && (j > 3) && hideSE) {
						// hide the NW
						icon = null;
					}
					if (icon != null) {
						g.drawImage(icon.getImage(), x, y, this);
					}
				}
			}

		}
	}

	/* (non-Javadoc)
	 * @see com.imaginot.chess.swing.ChessBoardListener#boardChanged(com.imaginot.chess.swing.ChessBoardEvent)
	 */
	public void boardChanged(ChessBoardEvent e) {
		ChessBoardModel theModel = (ChessBoardModel) e.getSource();
		if (theModel == model) {
			paint(getGraphics());
		}
	}
	/**
	 * @param b
	 */
	public void setHideNE(boolean b) {
		hideNE = b;
		repaint();
	}

	/**
	 * @param b
	 */
	public void setHideNW(boolean b) {
		hideNW = b;
		repaint();
	}

	/**
	 * @param b
	 */
	public void setHideSE(boolean b) {
		hideSE = b;
		repaint();
	}

	/**
	 * @param b
	 */
	public void setHideSW(boolean b) {
		hideSW = b;
		repaint();
	}

	/**
	 * @param b
	 */
	public void setWhiteBottom(boolean b) {
		whiteBottom = b;
		repaint();
	}

	public boolean getWhiteBottom() {
		return whiteBottom;
	}
	/**
	 * @return
	 */
	public boolean isHideNE() {
		return hideNE;
	}

	/**
	 * @return
	 */
	public boolean isHideNW() {
		return hideNW;
	}

	/**
	 * @return
	 */
	public boolean isHideSE() {
		return hideSE;
	}

	/**
	 * @return
	 */
	public boolean isHideSW() {
		return hideSW;
	}
	
	

	/**
	 * @return
	 */
	public boolean isHidePawns() {
		return hidePawns;
	}

	/**
	 * @return
	 */
	public boolean isHidePieces() {
		return hidePieces;
	}

	/**
	 * @param b
	 */
	public void setHidePawns(boolean b) {
		hidePawns = b;
		repaint();
	}

	/**
	 * @param b
	 */
	public void setHidePieces(boolean b) {
		hidePieces = b;
		repaint();
	}

}
